import { useCallback, useContext, useState } from 'react';
import Button from 'react-bootstrap/Button';
import Form from 'react-bootstrap/Form';
import Modal from 'react-bootstrap/Modal';
import PropTypes from 'prop-types';

import Web3Context from '../../contexts/Web3Context';
import Post from '../../utils/Post';

function AddPostModal({
  show, setShow, title, postToBlockchain, onSubmit,
}) {
  const {
    provider, DAO, account,
  } = useContext(Web3Context);
  const [content, setContent] = useState('');

  const handleClose = () => setShow(false);

  const handleSubmit = useCallback(async () => {
    // Upload content to API
    const post = new Post({ content, authors: [{ weightPPM: 1000000, authorAddress: account }] });
    // Include metamask signature
    await post.sign(provider, account);
    // Clear the input and hide the modal
    setContent('');
    setShow(false);
    // Write to API
    await post.write();
    // If requested, upload the hash to the blockchain
    if (postToBlockchain) {
      await post.publish(DAO, account);
    }
    // If requested, call callback
    if (onSubmit) {
      onSubmit(post);
    }
  }, [provider, DAO, account, content, setShow, postToBlockchain, onSubmit]);

  return (
    <Modal className="modal" show={show} onHide={handleClose}>
      <Modal.Header closeButton>
        <Modal.Title>{title}</Modal.Title>
      </Modal.Header>
      <Modal.Body>
        <Form>
          <Form.Group className="mb-3" controlId="addPost.content">
            <Form.Control
              as="textarea"
              rows={3}
              onChange={(e) => setContent(e.target.value)}
            />
          </Form.Group>
        </Form>

      </Modal.Body>
      <Modal.Footer>
        <Button variant="secondary" onClick={handleClose}>
          Close
        </Button>
        <Button variant="primary" onClick={handleSubmit}>
          Submit
        </Button>
      </Modal.Footer>
    </Modal>
  );
}

AddPostModal.propTypes = {
  show: PropTypes.bool.isRequired,
  setShow: PropTypes.func.isRequired,
  title: PropTypes.string,
  onSubmit: PropTypes.func,
  postToBlockchain: PropTypes.bool,
};

AddPostModal.defaultProps = {
  title: 'Add Post',
  onSubmit: undefined,
  postToBlockchain: false,
};

export default AddPostModal;
